#pragma rtGlobals=1		// Use modern global access method.

Menu "Particle Analysis"
"ROI tools", PA_ROI_tools()
"Single image", PA_singleImage()
"Image series", PA_imageSeries()
"PA to ROI Mask", PA_toRoiMask()
"Event series", PA_roiSeries()
"Event histogram",PA_RoiHistogram()
"Weibull Fit",PA_WeibullFit()
"Event correlation", PA_RoiCorrlation()
end

Proc PA_ROI_tools()
	// recall data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr):
	// recall ROI tools
	WMCreateImageROIPanel();
End

Function PA_singleImage()
	// set datafolder
	String dfStr=SIS_getDataFolderName() // where image is stored
	SetDataFolder root:$(dfStr)
	String imageStr=SIS_getImageStackName() // image name
	String winStr=SIS_getWinName() // nome della finestra
	NVAR layer = root:$(dfStr):layer
	//  input
	String objtypeStr="dark regions"
	variable objtype=2
	Prompt objtype,  "object type", popup "dark regions; bright regions;"
	variable mode=1
	Prompt mode, "threshold mode", popup "user level (0-255);halfway within ROI; V_avg of ROI; halfway within histogram;"
	variable thresh=10
	Prompt thresh, "threshold"
	variable minArea=8
	Prompt minArea, "minimum area"
	doPrompt  "input particle analysis", objtype, mode, thresh, minArea
	
	// if stack choose layer
	variable nr = dimSize($imageStr,2)
	if (V_flag==0)
		if (exists("M_ROIMask")==1) // the ROI wave exists otherwise quit
			// bright or dark
			if (objtype ==2)
				objtypeStr="bright regions"
			endif
			// calc ROI area
			waveStats/Q M_ROIMask 
			variable areaTotROIMask=V_npnts-V_sum
			// calc threshold
			if (nr > 1)
				ImageTransform/P=(layer)/PTYP=0 getPlane $imageStr // get layer
				ImageTransform convert2gray M_ImagePlane
			else			
				ImageTransform convert2gray $imageStr // creates binary image and store result in M_Image2grey
			endif
			
			if (mode > 1)
				ImageStats/R=M_ROIMask M_Image2Gray
				if (mode == 2)
					thresh = (V_max+V_min)/2
				endif
				if (mode == 3)
					thresh = V_rms
				endif
				if (mode == 4)
					ImageHistogram/R=M_ROIMask M_Image2Gray
					thresh = 0.5*(PA_histogramThreshold(W_ImageHist,0.03)+PA_histogramThreshold(W_ImageHist,0.97))
				endif
			endif
			if (objtype==1)
				ImageThreshold/M=0/T=(thresh)/Q/R=M_ROIMask M_Image2Gray
			else
				ImageThreshold/I/M=0/T=(thresh)/Q/R=M_ROIMask M_Image2Gray
			endif
			ImageAnalyzeParticles /W/Q/M=3/A=(minArea)/R=M_ROIMask/F stats, root:$(dfStr):M_ImageThresh
			PA_boundaryToFront(winStr,"W_BoundaryY","W_BoundaryX") //AppendToGraph/T root:$(dfStr):W_BoundaryY vs root:$(dfStr):W_BoundaryX
			// calculate and print total area particles within ROI
			variable areaParticles=sum(W_ImageObjArea)
			variable errorAreaParticles=sum(W_ImageObjPerimeter)
			String textBoxMessage="Object: "+objtypeStr+";\r"+"Threshold = "+num2Str(thresh)+"; Minimum Area = "+num2Str(minArea)+";\r"+"Total area = "+num2Str(areaParticles/areaTotROIMask*100)+"  "+num2Str(errorAreaParticles/areaTotROIMask*50)+" %"
			TextBox/C/N=areatext/A=LB "\\F'Arial'\\Z10"+textBoxMessage
			print ReplaceString("\r", textBoxMessage, " ")
		else
			print "please define ROI using ROI tools"
		endif
	endif
end

Function PA_imageSeries()
	// set datafolder
	String dfStr=SIS_getDataFolderName() // where image is stored
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName() // image name
	String winStr=SIS_getWinName() // nome della finestra
	// global variables
	NVAR layer = root:$(dfStr):layer
	SVAR selectedwaveStr=root:$(stackStr):selectedwaveStr
	wave wv=$selectedwaveStr
	wave M_imagePlane
	NVAR valdis = root:SIS:valdis
	//
	variable timeflag=0
	String unitsStr
	// 
	variable nlayers = dimSize($stackStr,2)
	// if nlayer == 1 call singleimage
	if (nlayers < 2)
		PA_singleImage()
	endif
	//  input
	String objtypeStr="dark regions"
	variable objtype=2
	Prompt objtype,  "object type", popup "dark regions; bright regions;"
	variable mode=1
	Prompt mode, "threshold mode", popup "user level (0-255);% of ROI; V_avg of ROI; % of histogram; relative to histogram maximum;"
	variable perc=0.33
	Prompt perc, "percentage level during threshold"
	variable thresh=50
	Prompt thresh, "threshold"
	variable upperthresh=255
	Prompt upperthresh, "upppr threshold"
	variable minArea=20
	Prompt minArea, "minimum area"
	variable from=0 //layer
	Prompt from, "from layer"
	variable to=nlayers-1
	Prompt to, "to layer"
	variable makeMovie=1
	Prompt makeMovie,  "make movie", popup "no; yes;"
	variable fps=5
	Prompt fps, "movie speed (frame/s): "
	doPrompt  "input particle analysis", objtype, mode, perc, thresh, upperthresh, minArea, from, to, makeMovie, fps
	if (V_flag==0)
		if (exists("M_ROIMask")==1) // the ROI wave exists otherwise quit
			if (makeMovie==2) // ask to save movie
				NewMovie /F=(fps)/L/O/Z/I as stackStr
				// define units
				if (stringmatch(selectedwaveStr,"pyro*")==1)
					unitsStr=" C"
				endif
				if (stringmatch(selectedwaveStr,"temp*")==1)
					unitsStr=" C"
				endif
				if (stringmatch(selectedwaveStr,"PMCH*")==1)
					unitsStr=" mbar"
				endif
				if (stringmatch(selectedwaveStr,"time*")==1)
					unitsStr=" s"
					timeflag=1
				endif
			endif
			// initialize waves
			String areaStr=dfStr+"_AR"
			String perimeterStr=dfStr+"_PE"
			String areaerrorStr=dfStr+"_ARE"
			String nparticlesStr=dfStr+"_NP"
			make/n=(nlayers)/O  $areaStr=NaN,   $areaerrorStr=NaN,  $perimeterStr=NaN, $nparticlesStr=NaN
			wave aw=$areaStr
			wave aew=$areaerrorStr
			wave pw=$perimeterStr
			wave npw=$nparticlesStr
			// initialize other variables
			if (objtype ==2)
				objtypeStr="bright regions"
			endif
			// calc ROI area
			waveStats/Q M_ROIMask 
			variable areaTotROIMask=V_npnts-V_sum
			// calc threshold for the selected layer
			ImageTransform/P=(layer)/PTYP=0 getPlane $stackStr // puts layer into M_ImagePlane
			M_imageplane[0,0]=0
			M_imageplane[0,1]=upperthresh
			M_imageplane=min(M_imageplane,upperthresh)
			ImageTransform convert2gray M_ImagePlane // creates binary image and store result in M_Image2grey
			variable hmax=NaN
			ImageStats/R=M_ROIMask M_Image2Gray
			if (mode > 1)
				if (mode == 2)
					thresh = perc*(V_max+V_min)
				endif
				if (mode == 3)
					thresh = V_rms
				endif
				if ((mode == 4) || (mode == 5))
					ImageHistogram/R=M_ROIMask M_Image2Gray
					thresh = perc*(PA_histogramThreshold(W_ImageHist,0.03)+PA_histogramThreshold(W_ImageHist,0.97))
					wavestats/Q W_ImageHist
					hmax=V_maxloc
				endif
			endif
			// loop
			variable i=from
			variable thresh_i
			String textBoxMessage=""
			variable unitstep= 1 
			if (to<from)
				unitstep=-1
			endif
			print from,to
			do
				// sho layer
				layer=i
				modifyImage $(stackStr) plane=(i)
				valdis=wv[i]
				ValDisplay vd,value=#num2Str(valdis)
				// layer operations
				ImageTransform/P=(i)/PTYP=0 getPlane $stackStr // get layer
				M_imageplane[0,0]=0
				M_imageplane[0,1]=upperthresh
				M_imageplane=min(M_imageplane,upperthresh)
				M_imageplane=max(M_imageplane,0)
				ImageTransform convert2gray M_ImagePlane
				// threshold
				if (mode == 5)
					ImageHistogram/R=M_ROIMask M_Image2Gray
					Smooth 4, W_ImageHist
					WaveStats/Q W_ImageHist
					thresh_i = thresh*V_maxloc/hmax
				else
					thresh_i=thresh
				endif
				// calculate threshold image for bright or dark particles
				if (objtype==1)
					ImageThreshold/M=0/T=(thresh_i)/Q/R=M_ROIMask M_Image2Gray
				else
					ImageThreshold/I/M=0/T=(thresh_i)/Q/R=M_ROIMask M_Image2Gray
				endif 
				ImageAnalyzeParticles /W/Q/M=3/A=(minArea)/R=M_ROIMask/F stats, root:$(dfStr):M_ImageThresh
				// update if i==from
				if (i==from)
					PA_boundaryToFront(winStr,"W_BoundaryY","W_BoundaryX") 
				endif
				// calculate and print total area particles within ROI
				npw[i]=V_NumParticles
				aw[i]=sum(W_ImageObjArea)/areaTotROIMask*100
				aew[i]=sum(W_ImageObjPerimeter)/areaTotROIMask*50
				pw[i]=sum(W_ImageObjPerimeter)/areaTotROIMask*100
				// print texbox on stack 
				if (makeMovie==1)
					textBoxMessage="i = "+num2Str(i)+"; Object: "+objtypeStr+";\r"+"Threshold = "+num2Str(thresh)+"; Minimum Area = "+num2Str(minArea)+";\r"+"Total area = "+num2Str(aw[i])+"  "+num2Str(pw[i])+" %"
					TextBox/C/N=areatext/A=LB "\\F'Arial'\\Z10"+textBoxMessage
					//print ReplaceString("\r", textBoxMessage, " ")
				else
					if (timeflag==1)
						sprintf textbox2Str "%*.*f" 4, 2, wv[i]/1000
					else
						sprintf textboxMessage "%*.*f" 4, 2, wv[i]
					endif
					TextBox/C/N=areatext/A=LB textboxMessage+unitsStr
				endif
				// update
				doUpdate
				// movie
				if (makeMovie==2) //  save movie frame
					AddMovieFrame
				endif
				// loop
				i+=unitstep
			while ((i<=abs(to)) && (i>-1))
			if (makeMovie==2) // close movie
				closeMovie
			endif
			// restore layer
			modifyImage $(stackStr) plane=(0)
			valdis=wv[0]
			ValDisplay vd,value=#num2Str(0)
			removeFromGraph W_BoundaryY
			TextBox/K/N=areatext
			doUpdate
			// display results if ...
			variable axis=2
			Prompt axis, "x axis", popup "image number;time;temperature (TC);temperature (PY);other;"
			doPrompt "axes label" axis
			if (V_flag==0)	
				String xStr
				String graphStr="PA"+dfStr
				String paxStr="PA"+dfStr+"_P"
				// image number
				if (axis == 1)
					Prompt xStr, "enter wave",popup WaveList("*NUM*",";","")
					doPrompt "bottom axis input wave" xStr
					if (V_flag==0)	
						duplicate/o $xStr $paxStr
						SIS_graphToFrontLeft(graphStr,areaStr,paxStr,"areaaxis")
						SIS_graphToFrontLeft(graphStr,nparticlesStr,paxStr,"particleaxis")
						Label bottom "\\F'Arial'\\Z16Image number"
					endif
				endif
				// time
				if (axis == 2)
					Prompt xStr, "enter wave",popup WaveList("ti*",";","")
					doPrompt "bottom axis input wave" xStr
					if (V_flag==0)	
						duplicate/o $xStr $paxStr
						wave paxw=$paxStr
						paxw/=1000 
						//SIS_xywaveToLeftAxis(graphStr,areaStr,paxStr)
						//SIS_xywaveToRightAxis(graphStr,nparticlesStr,paxStr)
						SIS_graphToFrontLeft(graphStr,areaStr,paxStr,"areaaxis")
						SIS_graphToFrontLeft(graphStr,nparticlesStr,paxStr,"particleaxis")
						Label bottom "\\F'Arial'\\Z16Time (s)"
					endif
				endif
				// temperature (TC)
				if (axis == 3)
					Prompt xStr, "enter wave",popup WaveList("te*",";","")
					doPrompt "bottom axis input wave" xStr
					if (V_flag==0)	
						duplicate/o $xStr $paxStr
						SIS_graphToFrontLeft(graphStr,areaStr,paxStr,"areaaxis")
						SIS_graphToFrontLeft(graphStr,nparticlesStr,paxStr,"particleaxis")
						Label bottom "\\F'Arial'\\Z16Temperature (C)"
					endif
				endif
				// temperature (Pyrometer)
				if (axis == 4)
					Prompt xStr, "enter wave",popup WaveList("py*",";","")
					doPrompt "bottom axis input wave" xStr
					if (V_flag==0)	
						duplicate/o $xStr $paxStr
						SIS_graphToFrontLeft(graphStr,areaStr,paxStr,"areaaxis")
						SIS_graphToFrontLeft(graphStr,nparticlesStr,paxStr,"particleaxis")
						Label bottom "\\F'Arial'\\Z16Temperature (C)"
					endif
				endif				
				// other
				if (axis == 5)
					Prompt xStr, "enter wave",popup WaveList("*",";","")
					doPrompt "bottom axis input wave" xStr
					if (V_flag==0)	
						duplicate/o $xStr $paxStr
						SIS_graphToFrontLeft(graphStr,areaStr,paxStr,"areaaxis")
						SIS_graphToFrontLeft(graphStr,nparticlesStr,paxStr,"particleaxis")
						Label bottom "\\F'Arial'\\Z16Other"
					endif
				endif
				ModifyGraph fSize=12
				Label areaaxis "\\F'Arial'\\Z14Area fraction (%)"
				Label particleaxis "\\F'Arial'\\Z14Number of particles"
				ModifyGraph mirror=1,axisEnab(areaaxis)={0.52,1},axisEnab(particleaxis)={0,0.48};DelayUpdate
				ModifyGraph freePos(areaaxis)=0,freePos(particleaxis)=0
				ModifyGraph lblPos(areaaxis)=60,lblPos(particleaxis)=60,lblLatPos=0
				ModifyGraph rgb($areaStr)=(65280,0,0);DelayUpdate
				ModifyGraph rgb($nparticlesStr)=(256,0,63232)
				ErrorBars $areaStr Y,wave=($areaerrorStr,$areaerrorStr)
			endif
		else
			print "please define ROI using ROI tools"
		endif
	endif
end

Function PA_toRoiMask()
	wave M_particle
	duplicate/o M_particle M_RoiMask
	M_RoiMask-=16
End

Function PA_roiSeries()
	// set datafolder
	String dfStr=SIS_getDataFolderName() // where image is stored
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName() // image name
	String winStr=SIS_getWinName() // nome della finestra
	// global variables
	wave W_xmin = root:$(dfStr):W_xmin
	wave W_xmax = root:$(dfStr):W_xmax
	wave W_ymin = root:$(dfStr):W_ymin
	wave W_ymax = root:$(dfStr):W_ymax
	variable npa = dimSize(W_xmin,0)	
	print "there are:"+num2str(npa)+" particles"
	// make roimask
	wave ws=$stackStr	
	variable np = dimSize(ws,0)	
	variable nq = dimSize(ws,1)
	variable nr = dimSize(ws,2)
	variable ir=0
	print "p.q:"+num2str(np)+","+num2str(nq)	
	make/n=(np,nq)/o/b/u PA_RoiMask=1
	make/n=(nr)/o wROI,dROI,wx=p
	make/n=(nr*10)/o w_eventTime,w_eventPosX,w_eventPosY,w_eventIndex
	variable ievent=0
//newImage/S=2 PA_RoiMask
//display wroi
	variable ipa=0
	variable thresh = 0
	variable j=0
	// input
	variable nsdev=3
	Prompt nsdev, "threshold", popup "1 sdev; 2 sdev; 3 sdev; 4 sdev; 5 sdev;"
	variable nrepeatedevents=5
	Prompt nrepeatedevents, "number of events per particle", popup "1; 2; 3; 4; 5;"
	variable axis=2
	Prompt axis, "x axis", popup "image number;time;temperature (TC);temperature (PY);other;"
	doPrompt "axes label" nsdev,nrepeatedevents,axis
	if (V_flag==0)	
		if (axis == 2)
			String xStr
			Prompt xStr, "enter wave",popup WaveList("ti*",";","")
			doPrompt "bottom axis input wave" xStr
			if (V_flag==0)	
				duplicate/o $xStr wx
				wx/=1000 
			endif
		endif	
	endif
	do
		// calculates the mask
		PA_RoiMask=1
		PA_RoiMask[W_xmin[ipa],W_xmax[ipa]][W_ymin[ipa],W_ymax[ipa]]=0
		// extract the beam
		ir=0
		do
			ImageStats/P=(ir)/R=PA_ROIMask ws 
			wROI[ir]=V_avg
			ir+=1
		while(ir<nr)
		// process the beam
		Smooth 2, wROI	
		Differentiate wROI /D =dROI
		wavestats/Q dROI
		doUpdate
		thresh = V_avg - nsdev*V_sdev
		j=0
		do 
			wavestats/Q dROI
			if (V_min < thresh)
				print "found event @: ",wx[V_minloc],V_minloc,V_min,thresh,(W_xmin[ipa]+W_xmax[ipa])/2,(W_ymin[ipa]+W_ymax[ipa])/2
				w_eventTime[ievent]=wx[V_minloc]
				w_eventPosX[ievent]=(W_xmin[ipa]+W_xmax[ipa])/2
				w_eventPosY[ievent]=(W_ymin[ipa]+W_ymax[ipa])/2
				w_eventIndex[ievent]=ievent
				dROI[V_minloc-3,V_minloc+3]=V_avg
				ievent+=1
			endif
			j+=1
		while (j < nrepeatedevents-1)
		ipa+=1
	while (ipa < npa)
	redimension/n=(ievent) w_eventTime, w_eventPosX, w_eventPosY, w_eventIndex
end

Function PA_RoiHistogram()
	variable channels=25
	Prompt channels, "number of channels"
	doPrompt  "input particle analysis ROI histogram", channels
	Make/N=(channels)/O w_eventTime_Hist
	Make/N=(channels)/O w_eventTime_Hist
	Histogram/B=1 w_eventTime,w_eventTime_Hist
	Display w_eventTime_Hist
	Integrate w_eventTime_Hist
	w_eventTime_Hist/=DimDelta(w_eventTime_Hist, 0 )
	Label left "\\F'Arial'\\Z16Total number of events ";DelayUpdate
	Label bottom "\\F'Arial'\\Z16Time (s)"
	ModifyGraph marker=8,mode(w_eventTime_Hist)=3,mode(fit_w_eventTime_Hist)=0
	//w_eventTime_Hist*=-1
	//Wavestats/Q w_eventTime_Hist
	//w_eventTime_Hist-=V_min
end


Function PA_RoiCorrlation()
	wave w_eventTime, w_eventPosX, w_eventPosY, w_eventIndex
	variable n2 = dimSize(w_eventTime,0)
	variable nsq= n2^2
	make/n=(nsq)/o distanceR, deltaT
	print n2
	variable k=0
	variable i,j
	for (i=0;i<n2;i+=1)
		for (j=0;j<n2;j+=1)
			distanceR[k]=sqrt( (w_eventPosX[i]-w_eventPosX[j])^2 + (w_eventPosY[i]-w_eventPosY[j])^2 )
			deltaT[k]= abs (w_eventTime[i]-w_eventTime[j]) 
			//print i,j,k
			k+=1				
		endfor
	endfor
	display distanceR vs deltaT
end

Function PA_WeibullFit()
	Wavestats/Q w_eventTime_Hist
	Make/n=4/o W_coef={V_max,V_maxloc/2,1,-60}
	Make/O/T/N=2 T_Constraints
	T_Constraints[0] = {"K2 > 0","K2 < 2"}
	FuncFit/H="1000"/NTHR=0 Weibull W_coef  w_eventTime_Hist /D /C=T_Constraints 
	ModifyGraph mode(fit_w_eventTime_Hist)=0,rgb(fit_w_eventTime_Hist)=(0,0,0)
end

Function PA_histogramThreshold(hw,thresh)
	wave hw
	variable thresh
	variable s=0,i=0
	variable tot= sum(hw)
	do
		s += hw[i]
		i+=1
	while( (s/tot) < thresh )
	return i
end

Function PA_boundaryToFront(graphNameStr,waveNameStr,wavexNameStr)
	String graphNameStr
	String waveNameStr
	String wavexNameStr
	graphNameStr= SIS_removeQuotes(graphNameStr)
	Dowindow $graphNameStr
	if (V_flag==0)
		Display
		DoWindow/C $graphNameStr
		appendToGraph/T $waveNameStr vs $wavexNameStr
		ModifyGraph width=240,height={Aspect,1}
		ModifyGraph mirror=1
	else
		DoWindow/F $graphNameStr
		String empty=WaveList(waveNameStr, ";", "WIN:"+graphNameStr)
		if (strlen(empty)==0)
			appendToGraph/T $waveNameStr vs $wavexNameStr
		endif
	endif
end

function Weibull(w,x) : FitFunc
	Wave w
	Variable x
       return w[0]*(1-(exp(-((x-w[3])/w[1])^w[2])))
End

function StretchedExp(w,x) : FitFunc
	Wave w
	Variable x
       return w[0]*(exp(-((x)/w[1])^w[2]))+w[3]
End
